#RNAseq pipeline
#Soukaina TIMOUMA et Alexandra MANCHENO FERRIS
#Master 2 Bioinformatique et Biologie des Systemes

#utilisateur donne input ( reads .fastq)
#recuperera RPKM
#il ne voit pas les etapes intermediaires
#fastq qualite
#fastq dans tophat => BAM
#BAM + htseq-count => .txt
#txt lit pour compter le RPKM
#usage : RNA_wrapper.py  <reads> <adapter>  <genome_ref_fasta> <miRbase_fasta> <gff_file> <output_file> <output_file2>


import subprocess
import sys
import os
import glob




output_dir = os.path.dirname(os.path.abspath(sys.argv[0]))

output_file = sys.argv[len(sys.argv)-2]
output_file2 = sys.argv[len(sys.argv)-1]

reads= sys.argv[1]
adapter = sys.argv[2]
genome_ref = sys.argv[3]
gtf_file = sys.argv[4]


#Quality analysis
control =  output_dir + "/control"
command_control = "mkdir " + control
if os.path.exists(control) == False:
        os.mkdir(control)
control_dir = control + "/"
command_fastqc = "fastqc --quiet -t 1 -o " + control_dir + " "+ reads
os.system(command_fastqc)

#
# # Output file
html = control_dir+ reads.split("/")[-1] + "_fastqc.html"
#
# ##Cut adaptater
#
# #Rename file for have the good extension
new_reads=reads+"_miRNA.fastq"
reads_cut = reads+".fastq"
command_rename = "scp " +reads + " "+ reads_cut
os.system(command_rename)


command_cut="cutadapt --quiet -b "+adapter+" -o "+new_reads+" "+reads_cut
#-b = the adapter can would be anywhere
os.system(command_cut)

# #Genome indexation
#
#
genome_ref_fasta = genome_ref+".fasta"
command_rename ="scp " + genome_ref + " " + genome_ref_fasta
os.system(command_rename)
index = output_dir +"/index"
if os.path.exists(index) == False:
      os.mkdir(index)
command_indexation = "bowtie2-build --quiet "+ genome_ref_fasta + " " + index +"/index &> /dev/null "


os.system(command_indexation)
#Mapping

command_mapping = "bowtie2 --quiet --no-sq --local --ma 1 -p 4 -q -x "+ index +"/index " + new_reads + " -S " + output_dir + "/accepted_hits.sam"
os.system(command_mapping)


# Count
#
command_count = "htseq-count -a 1 -q -t miRNA -i ID -f sam " +output_dir+"/accepted_hits.sam " + gtf_file + " > " + output_dir+ "/htseq_count.txt"
os.system(command_count)

#RPKM
#we read the htseq-count file to have the number of read for each gene
#we calculate the gene size with the information in the gff/gtf file
#we save this information into 2 dictionaries
#we calculate the RPKM
#lit le fichier de htseq_count on recupere le nombre de reads compte pour chaque gene
#lit le gtf pour avoir la longueur pour chaque gene

htseq_file = output_dir+ "/htseq_count.txt"

with open(htseq_file,'r') as htseq:
    lines= htseq.readlines()
    counts = list()
    total = 0
    htseq_count = {}
    for line in lines:
        if line.startswith("_") ==False:
            li=line.rstrip("\n")
            ID=li.split("\t")[0]
            nb_count = li.split("\t")[1]
            nb_count=int(nb_count)
            counts.append(nb_count)
            htseq_count[ID]=nb_count
    for i in counts:
        total = total + i
        #this is for don't have division by 0
        if total == 0:
            total = total +1
#
#print(total)
#
    with open(gtf_file,'r') as gtf:
        lines = gtf.readlines()
        gene = {}
        for line in lines:
            if line.startswith("#") == False:
                start = line.split("\t")[3]
                end = line.split("\t")[4]
                gene_length= int(end) - int(start)
                ID = line.split("\t")[8].split(";")[0].split("=")[1]
                gene[ID]=gene_length
#
RPKM_file = output_dir+"/RPKM.txt"
with open(RPKM_file,'w') as RPKM:
    header = "ID\tRPKM\n"
    RPKM.write(header)
    for key,values in htseq_count.items():
        for id,info in gene.items():
            if key == id:
                rpkm= values / ((info/1000.)*(total/1000000.))
                result = key + "\t" + str(rpkm) + "\n"
                RPKM.write(result)

#writing output RPKM html file for Galaxy
with open(output_file,'w') as output:
    output.write("<!DOCTYPE html>")
    output.write("<html>")
    output.write("<head>")
    output.write('<meta charset="utf-8" />')
    output.write("</head>")
    output.write("<body>")
    output.write("<p> <b> sRNAseq Pipeline Results : RPKM results </b> </p>")
    rpkm_file = output_dir+"/RPKM.txt"
    with open(RPKM_file) as lect:
        lignes = lect.readlines()
        for line in lignes:
            output.write('<p>'+ line + '</p>')
    output.write("</body>")
    output.write("</html>")
output.close()

#wirting output quality html file
with open(output_file2,'w') as output2:
    html_file = open(html, 'r')
    output2.write("<iframe>" + html_file.read() + "</iframe>")
    html_file.close()
output2.close()

